@@ -21,6 +21,7 @@
 #include "defaultdevice.h"
 #include "dsp.h"
 #include "dsp/manager.h"
+#include "stream/streammanager.h"
 #include <fitsio.h>
 
 #ifdef HAVE_WEBSOCKET
@@ -35,6 +36,9 @@
 #include <stdint.h>
 #include <mutex>
 #include <thread>
+#include <stream/streammanager.h>
+#include <connectionplugins/connectionserial.h>
+#include <connectionplugins/connectiontcp.h>
 
 //JM 2019-01-17: Disabled until further notice
 //#define WITH_EXPOSURE_LOOPING
@@ -71,17 +75,29 @@ class SensorInterface : public DefaultDevice
     public:
         enum
         {
-            SENSOR_CAN_ABORT                  = 1<<0,  /*!< Can the Sensor Integration be aborted?  */
-            SENSOR_HAS_STREAMING              = 1<<1,  /*!< Does the Sensor supports streaming?  */
-            SENSOR_HAS_SHUTTER                = 1<<2,  /*!< Does the Sensor have a mechanical shutter?  */
-            SENSOR_HAS_COOLER                 = 1<<3,  /*!< Does the Sensor have a cooler and temperature control?  */
-            SENSOR_HAS_DSP                    = 1<<4,
-            SENSOR_MAX_CAPABILITY             = 1<<5,  /*!< Does the Sensor have a cooler and temperature control?  */
+            SENSOR_CAN_ABORT                  = 1 << 0, /*!< Can the Sensor Integration be aborted?  */
+            SENSOR_HAS_STREAMING              = 1 << 1, /*!< Does the Sensor supports streaming?  */
+            SENSOR_HAS_SHUTTER                = 1 << 2, /*!< Does the Sensor have a mechanical shutter?  */
+            SENSOR_HAS_COOLER                 = 1 << 3, /*!< Does the Sensor have a cooler and temperature control?  */
+            SENSOR_HAS_DSP                    = 1 << 4,
+            SENSOR_MAX_CAPABILITY             = 1 << 5, /*!< Does the Sensor have a cooler and temperature control?  */
         } SensorCapability;
 
         SensorInterface();
         ~SensorInterface();
 
+
+        /**
+         * \enum SensorConnection
+         * \brief Holds the connection mode of the Sensor.
+         */
+        enum
+        {
+            CONNECTION_NONE   = 1 << 0, /** Do not use any connection plugin */
+            CONNECTION_SERIAL = 1 << 1, /** For regular serial and bluetooth connections */
+            CONNECTION_TCP    = 1 << 2  /** For Wired and WiFI connections */
+        } SensorConnection;
+
         bool initProperties();
         bool updateProperties();
         bool processNumber(const char *dev, const char *name, double values[], char *names[], int n);
@@ -96,7 +112,7 @@ class SensorInterface : public DefaultDevice
          * @brief getContinuumBufferSize Get allocated continuum buffer size to hold the Sensor integrationd stream.
          * @return allocated continuum buffer size to hold the Sensor integration stream.
          */
-        inline int getBufferSize()
+        inline int getBufferSize() const
         {
             return BufferSize;
         }
@@ -105,7 +121,7 @@ class SensorInterface : public DefaultDevice
          * @brief getIntegrationLeft Get Integration time left in seconds.
          * @return Integration time left in seconds.
          */
-        inline double getIntegrationLeft()
+        inline double getIntegrationLeft() const
         {
             return FramedIntegrationN[0].value;
         }
@@ -114,7 +130,7 @@ class SensorInterface : public DefaultDevice
          * @brief getIntegrationTime Get requested Integration duration for the Sensor device in seconds.
          * @return requested Integration duration for the Sensor device in seconds.
          */
-        inline double getIntegrationTime()
+        inline double getIntegrationTime() const
         {
             return integrationTime;
         }
@@ -151,7 +167,7 @@ class SensorInterface : public DefaultDevice
          * @brief getBPS Get Sensor depth (bits per sample).
          * @return bits per sample.
          */
-        inline int getBPS()
+        inline int getBPS() const
         {
             return BPS;
         }
@@ -166,8 +182,8 @@ class SensorInterface : public DefaultDevice
          * @param sendToClient If true (default), the element limits are updated and is sent to the
          * client. If false, the element limits are updated without getting sent to the client.
          */
-        void setMinMaxStep(const char *property, const char *element, double min, double max, double step,
-                           bool sendToClient = true);
+        virtual void setMinMaxStep(const char *property, const char *element, double min, double max, double step,
+                                   bool sendToClient = true);
 
         /**
          * @brief setBufferSize Set desired buffer size. The function will allocate memory
@@ -217,7 +233,7 @@ class SensorInterface : public DefaultDevice
         void setNAxis(int value);
 
         /**
-         * @brief setIntegrationExtension Set integration exntension
+         * @brief setIntegrationExtension Set integration extension
          * @param ext extension (fits, jpeg, raw..etc)
          */
         void setIntegrationFileExtension(const char *ext);
@@ -233,7 +249,7 @@ class SensorInterface : public DefaultDevice
         /**
          * @return True if Sensor is currently exposing, false otherwise.
          */
-        inline bool isCapturing()
+        inline bool isCapturing() const
         {
             return (FramedIntegrationNP.s == IPS_BUSY);
         }
@@ -255,6 +271,7 @@ class SensorInterface : public DefaultDevice
         /// Misc.
         /////////////////////////////////////////////////////////////////////////////
         friend class StreamManager;
+        friend class StreamManagerPrivate;
         /**
          * @brief StartStreaming Start live video streaming
          * @return True if successful, false otherwise.
@@ -276,19 +293,79 @@ class SensorInterface : public DefaultDevice
         virtual bool StartIntegration(double duration);
 
         /**
-         * \brief Uploads target Device exposed buffer as FITS to the client. Dervied classes should class
+         * \brief Uploads target Device exposed buffer as FITS to the client. Derived classes should class
          * this function when an Integration is complete.
          * @param targetDevice device that contains upload integration data
          * \note This function is not implemented in Sensor, it must be implemented in the child class
          */
         virtual bool IntegrationComplete();
 
-protected:
+        /** \brief perform handshake with device to check communication */
+        virtual bool Handshake();
+
+        /**
+         * @brief setSensorConnection Set Sensor connection mode. Child class should call this in the constructor before Sensor registers
+         * any connection interfaces
+         * @param value ORed combination of SensorConnection values.
+         */
+        void setSensorConnection(const uint8_t &value);
+
+        /**
+         * @return Get current Sensor connection mode
+         */
+        inline uint8_t getSensorConnection()
+        {
+            return sensorConnection;
+        }
+
+        /**
+         * \brief Add FITS keywords to a fits file
+         * \param fptr pointer to a valid FITS file.
+         * \param buf The buffer of the fits contents.
+         * \param len The length of the buffer.
+         * \note In additional to the standard FITS keywords, this function write the following
+         * keywords the FITS file:
+         * <ul>
+         * <li>EXPTIME: Total Integration Time (s)</li>
+         * <li>DATAMIN: Minimum value</li>
+         * <li>DATAMAX: Maximum value</li>
+         * <li>INSTRUME: Sensor Name</li>
+         * <li>DATE-OBS: UTC start date of observation</li>
+         * </ul>
+         *
+         * To add additional information, override this function in the child class and ensure to call
+         * SensorInterface::addFITSKeywords.
+         */
+        virtual void addFITSKeywords(fitsfile *fptr, uint8_t* buf, int len);
+
+        /** A function to just remove GCC warnings about deprecated conversion */
+        void fits_update_key_s(fitsfile *fptr, int type, std::string name, void *p, std::string explanation, int *status);
+
+        /** Return the connection file descriptor */
+        int getPortFD()
+        {
+            return PortFD;
+        }
+
+        /** Export the serial connection object */
+        Connection::Serial *getSerialConnection()
+        {
+            return serialConnection;
+        }
+
+        /** Export the TCP connection object */
+        Connection::TCP *getTcpConnection()
+        {
+            return tcpConnection;
+        }
+
+
+    protected:
 
         /**
          * @return True if Sensor has mechanical or electronic shutter. False otherwise.
          */
-        bool HasShutter()
+        bool HasShutter() const
         {
             return capability & SENSOR_HAS_SHUTTER;
         }
@@ -296,7 +373,7 @@ protected:
         /**
          * @return True if Sensor has cooler and temperature can be controlled. False otherwise.
          */
-        bool HasCooler()
+        bool HasCooler() const
         {
             return capability & SENSOR_HAS_COOLER;
         }
@@ -304,17 +381,25 @@ protected:
         /**
          * @return True if Sensor can abort Integration. False otherwise.
          */
-        bool CanAbort()
+        bool CanAbort() const
         {
             return capability & SENSOR_CAN_ABORT;
         }
 
         /**
-         * @return  True if the Sensor supports live video streaming. False otherwise.
+         * @return  True if the Sensor wants DSP processing. False otherwise.
          */
         bool HasDSP()
         {
-            return capability & SENSOR_HAS_DSP;
+            if (capability & SENSOR_HAS_DSP)
+            {
+                if(DSP.get() == nullptr)
+                {
+                    DSP.reset(new DSP::Manager(this));
+                }
+                return true;
+            }
+            return false;
         }
 
         /**
@@ -322,7 +407,16 @@ protected:
          */
         bool HasStreaming()
         {
-            return capability & SENSOR_HAS_STREAMING;
+            if (capability & SENSOR_HAS_STREAMING)
+            {
+                if(Streamer.get() == nullptr)
+                {
+                    Streamer.reset(new StreamManager(this));
+                    Streamer->initProperties();
+                }
+                return true;
+            }
+            return false;
         }
 
         /**
@@ -334,7 +428,7 @@ protected:
         }
 
         /**
-         * @brief SetCapability Set the Sensor capabilities. Al fields must be initilized.
+         * @brief SetCapability Set the Sensor capabilities. Al fields must be initialized.
          * @param cap pointer to SensorCapability struct.
          */
         void SetCapability(uint32_t cap);
@@ -357,10 +451,6 @@ protected:
         IBLOB FitsB;
         IBLOBVectorProperty FitsBP;
 
-        //  We are going to snoop these from a telescope
-        INumberVectorProperty EqNP;
-        INumber EqN[2];
-
         ITextVectorProperty ActiveDeviceTP;
         IText ActiveDeviceT[4] {};
 
@@ -405,13 +495,20 @@ protected:
          */
         virtual bool saveConfigItems(FILE *fp);
 
-        double primaryAperture;
-        double primaryFocalLength;
         bool InIntegration;
 
-        double Lat, Lon, El;
+        INumberVectorProperty EqNP;
+        INumber EqN[2];
         double RA, Dec;
 
+        INumberVectorProperty LocationNP;
+        INumber LocationN[3];
+        double Latitude, Longitude, Elevation;
+
+        INumberVectorProperty ScopeParametersNP;
+        INumber ScopeParametersN[4];
+        double primaryAperture, primaryFocalLength;
+
         bool AutoLoop;
         bool SendIntegration;
         bool ShowMarker;
@@ -435,8 +532,15 @@ protected:
 
         std::unique_ptr<StreamManager> Streamer;
         std::unique_ptr<DSP::Manager> DSP;
+        Connection::Serial *serialConnection = NULL;
+        Connection::TCP *tcpConnection       = NULL;
+
+        /// For Serial & TCP connections
+        int PortFD = -1;
 
     private:
+        bool callHandshake();
+        uint8_t sensorConnection = CONNECTION_NONE;
 
         int BPS;
         /// # of Axis
@@ -445,35 +549,14 @@ protected:
         uint8_t *Buffer;
         int BufferSize;
         double integrationTime;
-        timespec startIntegrationTime;
+        double startIntegrationTime;
         char integrationExtention[MAXINDIBLOBFMT];
 
         bool uploadFile(const void *fitsData, size_t totalBytes, bool sendIntegration, bool saveIntegration);
         void getMinMax(double *min, double *max, uint8_t *buf, int len, int bpp);
         int getFileIndex(const char *dir, const char *prefix, const char *ext);
-        /**
-         * \brief Add FITS keywords to a fits file
-         * \param fptr pointer to a valid FITS file.
-         * \param targetDevice The target device to extract the keywords from.
-         * \param blobIndex The blob index of this FITS (0: continuum, 1: spectrum, 2: timedev).
-         * \note In additional to the standard FITS keywords, this function write the following
-         * keywords the FITS file:
-         * <ul>
-         * <li>EXPTIME: Total Integration Time (s)</li>
-         * <li>DATAMIN: Minimum value</li>
-         * <li>DATAMAX: Maximum value</li>
-         * <li>INSTRUME: Sensor Name</li>
-         * <li>DATE-OBS: UTC start date of observation</li>
-         * </ul>
-         *
-         * To add additional information, override this function in the child class and ensure to call
-         * Receiver::addFITSKeywords.
-         */
-        virtual void addFITSKeywords(fitsfile *fptr, uint8_t* buf, int len);
 
         bool IntegrationCompletePrivate();
         void* sendFITS(uint8_t* buf, int len);
-        /** A function to just remove GCC warnings about deprecated conversion */
-        void fits_update_key_s(fitsfile *fptr, int type, std::string name, void *p, std::string explanation, int *status);
 };
 }
